package com.qmth.teachcloud.common.threadPool;//package com.qmth.themis.business.threadPool;

import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @Description: 线程池应用配置
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/3/21
 */
@Configuration
public class DynamicMyThreadPool extends ThreadPoolTaskExecutor {
    private final static Logger log = LoggerFactory.getLogger(DynamicMyThreadPool.class);
    private DynamicMyThreadPool dynamicThreadPoolTaskExecutor = null;
    private static Object poolSizeMonitor = new Object();
    private static final int cpuNum = Runtime.getRuntime().availableProcessors();
    private static final int initCoreSize = 1;

    /**
     * 线程池初始化
     * 主要做了两件事 1 初始化线程池 2 初始化监控线程
     */
    @Bean
    @Primary
    public Executor taskThreadPool() {
        synchronized (this.poolSizeMonitor) {
            return this.createThreadPool();
        }
    }

    /**
     * 刷新线程池状态
     */
    public void refresh(int corePoolSize) {
        synchronized (this.poolSizeMonitor) {
            log.info(">>>>>>>>>>>> 开始刷新线程池参数 >>>>>>>>>>>");
            if (!this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().isShutdown() && !this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().isTerminated()) {
                this.dynamicThreadPoolTaskExecutor.setCorePoolSize(corePoolSize);
                this.dynamicThreadPoolTaskExecutor.setMaxPoolSize(this.dynamicThreadPoolTaskExecutor.getCorePoolSize() * 2);
                log.info(">>>>>>>>>>>> 线程池参数刷新完毕 >>>>>>>>>>>");
            } else {
                log.error(">>>>>>>>>>>> 线程池状态异常，不可刷新 >>>>>>>>>>>>");
//                throw new RuntimeException("线程池状态异常，不可刷新");
            }
        }
    }

    public void shutDown() {
        synchronized (this.poolSizeMonitor) {
            DynamicMyThreadPool executor = this.getExecutor();
            executor.shutdown();
            if (executor.getThreadPoolExecutor().isShutdown()) {
                executor = null;
                log.warn(">>>>>>>>>>>> 线程池已停止 >>>>>>>>>>>>");
            }
        }
    }

    public DynamicMyThreadPool getExecutor() {
        return this.dynamicThreadPoolTaskExecutor;
    }

    @Override
    public String toString() {
        int rate = this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getQueue().size() + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getActiveCount();
        BigDecimal total = SystemConstant.PERCENT.add(new BigDecimal(this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getActiveCount()));
        BigDecimal percent = new BigDecimal(rate).divide(total, 2, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP);
        return "当前线程池使用率：" + percent + "%" +
                ", 核心线程数：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getCorePoolSize() +
                ", 最大线程数：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getMaximumPoolSize() +
                ", 线程数量：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getPoolSize() +
                ", 队列数量：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getQueue().size() +
                ", 活跃数量：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getActiveCount() +
                ", 已完成数量：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getCompletedTaskCount() +
                ", 总共数量：" + this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getTaskCount();
    }

    public boolean getStatus() {
        return !this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().isShutdown() && !this.dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().isTerminated();
    }

    public Executor reLoad() {
        synchronized (this.poolSizeMonitor) {
            this.dynamicThreadPoolTaskExecutor = null;
            log.info(">>>>>>>>>>>> 准备重新创建线程池 >>>>>>>>>>>>", this.getThreadNamePrefix());
            return this.createThreadPool();
        }
    }

    protected Executor createThreadPool() {
        if (Objects.isNull(dynamicThreadPoolTaskExecutor)) {
            log.info(">>>>>>>>>>>> thread-pool【{}】start init >>>>>>>>>>>>", this.getThreadNamePrefix());
            this.dynamicThreadPoolTaskExecutor = new DynamicMyThreadPool();
            this.dynamicThreadPoolTaskExecutor.setCorePoolSize(initCoreSize);//核心线程数
            this.dynamicThreadPoolTaskExecutor.setMaxPoolSize(this.dynamicThreadPoolTaskExecutor.getCorePoolSize() * 2);//最大线程数
            this.dynamicThreadPoolTaskExecutor.setKeepAliveSeconds(SystemConstant.THREAD_POOL_KEEP_ALIVE_SECONDS);//线程空闲时间
            this.dynamicThreadPoolTaskExecutor.setQueueCapacity(SystemConstant.THREAD_POOL_QUEUE_CAPACITY);//队列容量
            this.dynamicThreadPoolTaskExecutor.setThreadNamePrefix(SystemConstant.THREAD_POOL_NAME);
            this.dynamicThreadPoolTaskExecutor.setAllowCoreThreadTimeOut(true);//设置是否允许核心线程超时。若允许，核心线程超时后，会被销毁。默认为不允许（fasle）
            this.dynamicThreadPoolTaskExecutor.setWaitForTasksToCompleteOnShutdown(true);//设置shutdown时是否等到所有任务完成再真正关闭
            this.dynamicThreadPoolTaskExecutor.setAwaitTerminationSeconds(60);//当setWaitForTasksToCompleteOnShutdown(true)时，setAwaitTerminationSeconds 设置在 shutdown 之后最多等待多长时间后再真正关闭线程池
            // rejection-policy：当pool已经达到max size的时候，如何处理新任务
            // CALLER_RUNS：不在新线程中执行任务，而是由调用者所在的线程来执行
            this.dynamicThreadPoolTaskExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
            this.dynamicThreadPoolTaskExecutor.initialize();
        }
        return dynamicThreadPoolTaskExecutor;
    }

    public DynamicMyThreadPool getDynamicThreadPoolTaskExecutor() {
        return dynamicThreadPoolTaskExecutor;
    }

    public int getCoreThreadSize() {
        return dynamicThreadPoolTaskExecutor.getThreadPoolExecutor().getCorePoolSize();
    }

    public int getCpuNum() {
        return cpuNum;
    }
}
