package com.qmth.teachcloud.common.util;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.core.fss.service.FileStoreService;
import com.qmth.boot.core.fss.store.FileStore;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.LocalCatalogEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.time.Duration;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 文件存储工具类
 * @Author: CaoZixuan
 * @Date: 2021-08-17
 */
@Component
public class FileStoreUtil {
    private final static Logger log = LoggerFactory.getLogger(FileStoreUtil.class);

    @Resource
    private FileStoreService fileStoreService;

    @Resource
    private DictionaryConfig dictionaryConfig;

    /**
     * 上传文件到本地
     *
     * @param inputStream 流
     * @param finalFile   最终文件
     * @param catalogType 文件类型
     * @throws Exception 异常
     */
    public void copyInputStreamToFile(InputStream inputStream, File finalFile, String md5, LocalCatalogEnum catalogType) throws Exception {
        String dirName = FileUtil.replaceSplit(finalFile.getPath());
        this.localUpload(dirName, inputStream, md5, catalogType);
    }

    /**
     * 上传文件到本地
     *
     * @param dirName     上传到地址
     * @param inputStream 流
     * @param catalogType 文件
     */
    public void localUpload(String dirName, InputStream inputStream, String md5, LocalCatalogEnum catalogType) throws Exception {
        log.info("ossUpload is come in");
        String configPath = "";
        switch (catalogType) {
            case LOCAL_FILE:
                configPath = dictionaryConfig.fssPublicDomain().getConfig();
                break;
            case LOCAL_PDF:
                configPath = dictionaryConfig.fssPrivateDomain().getConfig();
                break;
            default:
                break;
        }
        dirName = dirName.replaceAll(configPath, "");
        fileStoreService.getFileStore(catalogType.getType()).write(dirName, inputStream, md5);
        log.info("dirName:{}", dirName);
    }

    /**
     * 上传文件
     *
     * @param dirName     上传到地址
     * @param inputStream 流
     * @param type        fileStore类型
     */
    public void ossUpload(String dirName, InputStream inputStream, String md5, String type) throws Exception {
        log.info("ossUpload is come in");
        fileStoreService.getFileStore(type).write(dirName, inputStream, md5);
        log.info("dirName:{}", dirName);
    }

//    /**
//     * 上传文件到本地
//     *
//     * @param dirName     上传到地址
//     * @param inputStream 流
//     * @param catalogType 文件
//     */
//    public void localUpload(String dirName, InputStream inputStream, String md5, LocalCatalogEnum catalogType) throws Exception {
//        log.info("ossUpload is come in");
//        String configPath = "";
//        switch (catalogType) {
//            case LOCAL_FILE:
//                configPath = dictionaryConfig.fssLocalFileDomain().getConfig();
//                break;
//            case LOCAL_PDF:
//                configPath = dictionaryConfig.fssLocalPdfDomain().getConfig();
//                break;
//            default:
//                break;
//        }
//        dirName = dirName.replaceAll(configPath, "");
//        fileStoreService.getFileStore(catalogType.getType()).write(dirName, inputStream, md5);
//        log.info("dirName:{}", dirName);
//    }

//    /**
//     * 上传文件到本地
//     *
//     * @param inputStream 流
//     * @param finalFile   最终文件
//     * @param catalogType 文件类型
//     * @throws Exception 异常
//     */
//    public void copyInputStreamToFile(InputStream inputStream, File finalFile, String md5, LocalCatalogEnum catalogType) throws Exception {
//        String dirName = finalFile.getPath().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
//        this.localUpload(dirName, inputStream, md5, catalogType);
//    }

    /**
     * 上传文件
     *
     * @param dirName 上传到地址
     * @param file    文件
     * @param type    fileStore类型
     */
    public void ossUpload(String dirName, File file, String md5, String type) throws Exception {
        log.info("ossUpload is come in");
        fileStoreService.getFileStore(type).write(dirName, new FileInputStream(file), md5);
        log.info("dirName:{}", dirName);
    }

    /**
     * 上传文件
     *
     * @param dirName 上传到地址
     * @param content 文件
     * @param type    fileStore类型
     */
    public void ossUpload(String dirName, String content, String type) throws Exception {
        log.info("ossUpload is come in");
        fileStoreService.getFileStore(type).write(dirName, new ByteArrayInputStream(content.getBytes()), DigestUtils.md5Hex(new ByteArrayInputStream(content.getBytes())));
        log.info("dirName:{}", dirName);
    }

    /**
     * 从文件存储上下载文件到本地
     *
     * @param dirName   文件地址
     * @param localPath 本地路径
     * @param type      fileStore类型
     * @throws Exception 异常
     */
    public File ossDownload(String dirName, String localPath, String type) throws Exception {
        log.info("ossDownload is come in");
        return this.saveLocal(fileStoreService.getFileStore(type).read(dirName), localPath);
    }

    /**
     * 从文件存储上下载文件到本地
     *
     * @param dirName   文件地址
     * @param localFile 本地路径
     * @param type      fileStore类型
     * @throws Exception 异常
     */
    public File ossDownload(String dirName, File localFile, String type) throws Exception {
        log.info("ossDownload is come in");
        InputStream inputStream = fileStoreService.getFileStore(type).read(dirName);
        FileUtils.copyInputStreamToFile(inputStream, localFile);
        return localFile;
    }

    /**
     * 从文件存储上下载文件到byte[]
     *
     * @param objectName 文件地址
     * @param type       fileStore类型
     * @throws Exception 异常
     */
    public byte[] ossDownload(String objectName, String type) throws Exception {
        log.info("oss Download is come in");
        return IOUtils.toByteArray(fileStoreService.getFileStore(type).read(objectName));
    }

    /**
     * 从文件存储上下载文件到InputStream
     *
     * @param objectName 文件地址
     * @param type       fileStore类型
     * @throws Exception 异常
     */
    public InputStream ossDownloadIs(String objectName, String type) throws Exception {
        log.info("oss Download is come in");
        return fileStoreService.getFileStore(type).read(objectName);
    }

    /**
     * 获取文件访问url
     *
     * @param objectPath 文件路径
     * @param type       文件上传的类型
     * @return
     */
    public String getPrivateUrl(String objectPath, String type) {
        String server = null;
        if ("public".equals(type)) {
            server = dictionaryConfig.fssPublicDomain().getServer();
            return server + SystemConstant.ORG_SPLIT + objectPath;
        } else if ("private".equals(type)) {
            Boolean oss = dictionaryConfig.sysDomain().isOss();
            if (Objects.nonNull(oss) && oss) {
                FileStore fileStore = fileStoreService.getFileStore(type);
                return fileStore.getServerUrl(objectPath, Duration.ofMinutes(5L));
            } else {
                if (objectPath.startsWith(SystemConstant.ORG_SPLIT)) {
                    return dictionaryConfig.fssPrivateDomain().getServer() + objectPath;
                } else {
                    return dictionaryConfig.fssPrivateDomain().getServer() + SystemConstant.ORG_SPLIT + objectPath;
                }
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件存储store类型不存在");
        }
    }

    /**
     * 获取文件访问url
     *
     * @param objectPath 文件路径
     * @param type       文件上传的类型
     * @return
     */
    public String getPrivateUrlPrefix(String objectPath, String type) {
        String server = null;
        if ("public".equals(type)) {
            server = dictionaryConfig.fssPublicDomain().getServer();
            return server + SystemConstant.ORG_SPLIT;
        } else if ("private".equals(type)) {
            Boolean oss = dictionaryConfig.sysDomain().isOss();
            if (Objects.nonNull(oss) && oss) {
                FileStore fileStore = fileStoreService.getFileStore(type);
                return fileStore.getServer() + SystemConstant.ORG_SPLIT;
            } else {
                if (objectPath.startsWith(SystemConstant.ORG_SPLIT)) {
                    return dictionaryConfig.fssPrivateDomain().getServer();
                } else {
                    return dictionaryConfig.fssPrivateDomain().getServer() + SystemConstant.ORG_SPLIT;
                }
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件存储store类型不存在");
        }
    }

    /**
     * 获取文件访问url
     *
     * @param objectPath 文件路径
     * @param type       文件上传的类型
     * @return
     */
    public String getPrivateUrlExpire(String objectPath, String type, Boolean isExpire) {
        String server = null;
        if ("public".equals(type)) {
            server = dictionaryConfig.fssPublicDomain().getServer();
            return server + SystemConstant.ORG_SPLIT + objectPath;
        } else if ("private".equals(type)) {
            Boolean oss = dictionaryConfig.sysDomain().isOss();
            if (Objects.nonNull(oss) && oss) {
                FileStore fileStore = fileStoreService.getFileStore(type);
                if (isExpire) {
                    return fileStore.getServerUrl(objectPath, Duration.ofMinutes(5L));
                } else {
                    return fileStore.getServerUrl(objectPath, Duration.ofMinutes(300L));
                }
            } else {
                return dictionaryConfig.fssPrivateDomain().getServer() + SystemConstant.ORG_SPLIT + objectPath;
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件存储store类型不存在");
        }
    }

    /**
     * 根据数据库文件路径判断文件上传类型
     *
     * @param path 路径
     * @return 类型
     */
    public UploadFileEnum getUploadEnumByPath(String path) {
        path = FileUtil.replaceSplit(path);
        String target = path.substring(0, path.indexOf('/'));
        return UploadFileEnum.valueOf(target.toUpperCase());
    }

    /**
     * 文件存在某本地路径
     *
     * @param inputStream 输入流
     * @param dirPath     本地路径
     * @return 存好的文件
     * @throws IOException 异常
     */
    public File saveLocal(InputStream inputStream, String dirPath) throws IOException {
        String fileName = dirPath.substring(dirPath.lastIndexOf(File.separator) + 1);
        String parentPath = dirPath.substring(0, dirPath.lastIndexOf(File.separator)) + File.separator;
        String name = fileName.substring(0, fileName.lastIndexOf('.'));
        String suffix = fileName.substring(fileName.lastIndexOf('.'));

        File desFile = buildSingleFileName(parentPath, name, suffix, 0);
        if (!desFile.exists()) {
            desFile.getParentFile().mkdirs(); //目标文件目录不存在的话需要创建目录
            desFile.createNewFile();
        }
//        if (desFile.length() > 0) {
//            return desFile;
//        }
        try {
            FileUtils.copyInputStreamToFile(inputStream, desFile);
            return desFile;
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /**
     * 构建唯一的文件名称
     *
     * @param path     文件路径
     * @param fileName 文件名称
     * @param suffix   文件后缀
     * @param index    当前下标，初次为0
     * @return 唯一的文件名称
     */
    public static File buildSingleFileName(String path, String fileName, String suffix, Integer index) {
        File file;
        //下标不等于0开始拼后缀
        if (index != 0) {
            file = new File(path + fileName + "(" + index + ")" + suffix);
        } else {
            file = new File(path + fileName + suffix);
        }
        //判断文件是否存在 文件不存在退出递归
        if (file.isFile()) {
            //每次递归给下标加1
            file = buildSingleFileName(path, fileName, suffix, ++index);
        }
        return file;
    }

    public String uploadFile(MultipartFile file, String md5, UploadFileEnum type) {
        JSONObject jsonObject;
        try {
            String fileMd5 = DigestUtils.md5Hex(file.getBytes());
            log.info("fileMd5:{}", fileMd5);
            log.info("md5:{}", md5);
            if (!Objects.equals(fileMd5, md5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }

            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss && (Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig())) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
            }
            String format = "." + FilenameUtils.getExtension(file.getOriginalFilename());
            stringJoiner = SystemConstant.getDirName(stringJoiner, type, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(format);

            jsonObject = new JSONObject();
            String dirName = FileUtil.replaceSplit(stringJoiner.toString());
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {//上传至oss
                this.ossUpload(dirName, file.getInputStream(), md5, type.getFssType());
                jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, dirName);
            } else {//上传至服务器
                this.localUpload(dirName, file.getInputStream(), md5, LocalCatalogEnum.LOCAL_FILE);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, dirName);
            }
            jsonObject.put(SystemConstant.UPLOAD_TYPE, type);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("文件上传失败");
        }

        return jsonObject.toJSONString();
    }
}
